"""
$Id: PSCRelease.py,v 1.11 2005/04/01 02:31:47 optilude Exp $
"""

import re

from random import random
from DateTime import DateTime

from Acquisition import aq_inner
from Acquisition import aq_parent

from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import registerType
from Products.Archetypes.public import DisplayList
from Products.Archetypes.public import OrderedBaseFolder

from Products.PloneSoftwareCenter.config import PROJECTNAME
from Products.PloneSoftwareCenter.utils import folder_modify_fti
from Products.PloneSoftwareCenter.content.schemata import PSCReleaseSchema

def modify_fti(fti):
    folder_modify_fti(fti, allowed=('PSCFile','PSCFileLink'))


class PSCRelease(OrderedBaseFolder):
    """A Package contains Releases, which have Files."""

    __implements__ = (OrderedBaseFolder.__implements__,)

    archetype_name = 'Software Release'
    immediate_view = default_view = 'psc_release_view'
    content_icon = 'download_icon.gif'
    schema = PSCReleaseSchema

    security = ClassSecurityInfo()

    actions = (
        {
            'id': 'view',
            'name': 'View',
            'action': 'string:${object_url}/psc_release_view',
            'permissions': (CMFCorePermissions.View,),
        },
    )

    typeDescMsgId = 'description_edit_release'
    typeDescription = ('An Release contains details about a planned '
                       'or completed release of the package. Releases '
                       'may be related to specific Improvement Proposals, '
                       'to generate a roadmap. You add the downloadable '
                       'files inside the release.')

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 
                                'setRepository')
    def setRepository(self, value):
        """Set the repository branch, stripping off whitespace and any trailing
        slashes
        """
        value = value.strip()
        while value.endswith('/'):
            value = value[:-1]
        self.getField('repository').set(self, value)

    security.declareProtected(CMFCorePermissions.View, 'isOutdated')
    def isOutdated(self):
        """Return true if this release is no longer supported."""
        currentVersions = self.getCurrentVersions()
        if currentVersions and self.getId() not in currentVersions:
            return True
        else:
            return False
            
    security.declareProtected(CMFCorePermissions.View, 'isPreferredMaturity')
    def isPreferredMaturity(self):
        """Return true if this release is of the preferred maturity."""
        preferredMaturity = self.getPreferredMaturity()
        if preferredMaturity and self.getMaturity() == preferredMaturity:
            return True
        else:
            return False

    security.declareProtected(CMFCorePermissions.View, 'getMaturityVocab')
    def getMaturityVocab(self):
        """Get the available maturity states from the parent package area
        via acqusition.
        """
        return self.getAvailableMaturitiesAsDisplayList()

    security.declareProtected(CMFCorePermissions.View, 'getCompatibilityVocab')
    def getCompatibilityVocab(self):
        """Get the available compatability versions from the parent package area
        via acqusition.
        """
        return self.getAvailableVersionsAsDisplayList()

    security.declareProtected(CMFCorePermissions.View, 'getLicenseVocab')
    def getLicenseVocab(self):
        """Get the available licenses from the parent package area via
         acqusition.
        """
        return self.getAvailableLicensesAsDisplayList()

    security.declarePublic('generateTitle')
    def generateTitle(self):
        """Generate the title of the release from the project name +
        the version
        """
        # The first time this is called (it's called like two dozen times
        # when a release is created...) we don't seem to have an acquisition
        # context, so the call to aq_inner fails. Thus, we fall back on a
        # hardcoded string if the parent title lookup fails.
        try:
            version = self.getId()
            parentTitle = self.aq_inner.aq_parent.aq_parent.Title()
        except:
            version = '?'
            parentTitle = '?'
        return "%s %s" % (parentTitle, version)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'validate_id')
    def validate_id(self, value):
        """Validate the id field, ensuring a valid web address was
        entered.
        """
        if not value:
            return "Please provide a version number"
        if re.search (r'[^\w.-]', value):
            return ('Please only use numbers, letters, underscores (_), '
                    'dashes (-) and periods (.) in the version string, no '
                    'other punctuation characters or whitespace')
        else:
            return None

    security.declareProtected(CMFCorePermissions.View,
                              'getRelatedFeaturesVocab')
    def getRelatedFeaturesVocab(self):
        """Get list of PLIPs possible to add to this release."""
        catalog = getToolByName(self, 'portal_catalog')
        search = catalog.searchResults(portal_type = 'PSCImprovementProposal',
                                       path = '/'.join(self.getPhysicalPath()[:-2]))
        
        items = [s for s in search]
        items.sort(lambda x, y: cmp(int(x.getId), int(y.getId)))
        return DisplayList([(i.UID, i.Title) for i in items])

    security.declareProtected(CMFCorePermissions.View, 'getRelatedFeatures')
    def getRelatedFeatures(self, review_state=None):
        """Get list of PLIPs associated with this release. Give a review state
        to return only features in this state.
        """
        wftool = getToolByName(self, 'portal_workflow')
        items = self.getField('relatedFeatures').get(self)
        if review_state:
            items = [i for i in items if wftool.getInfoFor(i, 'review_state') == review_state]
        items.sort(lambda x, y: cmp(int(x.getId()), int(y.getId())))
        return items   

    security.declareProtected(CMFCorePermissions.AddPortalContent, 'generateUniqueId')
    def generateUniqueId(self, type_name):
        """We don't want the behavior of release folder, choosing on
        versions. So use the standard Plone behavior.
        """
        now = DateTime()
        time = '%s.%s' % (now.strftime('%Y-%m-%d'), str(now.millis())[7:])
        rand = str(random())[2:6]
        prefix = ''
        suffix = ''
        if type_name is not None:
            prefix = type_name.replace(' ', '_') + '.'
        prefix = prefix.lower()
        return prefix + time + rand + suffix


registerType(PSCRelease, PROJECTNAME)
